@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2010, 2018 Free Software Foundation, Inc.                  */
+/*  Copyright (C) 2010-2020 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -22,17 +22,21 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #include <getopt.h>
 
 #include <dwg.h>
 #include "../src/common.h"
 #include "suffix.inc"
 
-static int opts = 1;
+// avoid the slow fork loop, for afl-clang-fast
+#ifdef __AFL_COMPILER
+static volatile char *__afl_persistent_sig = "##SIG_AFL_PERSISTENT##";
+#endif
 
+static int opts = 1;
 static int help (void);
-// int verbosity(int argc, char **argv, int i, unsigned int *opts);
-//#include "common.inc"
 
 static int
 usage (void)
@@ -91,6 +95,7 @@ main (int argc, char *argv[])
   char *filename_in;
   const char *version = NULL;
   char *filename_out = NULL;
+  int free_fnout = 0;
   Dwg_Version_Type dwg_version;
   BITCODE_BL num_objects;
   int c;
@@ -195,7 +200,12 @@ main (int argc, char *argv[])
   i = optind;
 
   memset (&dwg, 0, sizeof (Dwg_Data));
-  dwg.opts = opts;
+  dwg.opts = opts & 0xf;
+
+#ifdef __AFL_HAVE_MANUAL_CONTROL
+  __AFL_INIT();
+  while (__AFL_LOOP(1000)) {
+#endif
 
   filename_in = argv[i];
   if (!filename_in)
@@ -208,11 +218,14 @@ main (int argc, char *argv[])
       if (argc > i + 1)
         filename_out = argv[i + 1];
       else
-        filename_out = suffix (filename_in, "-rewrite.dwg");
+        {
+          free_fnout = 1;
+          filename_out = suffix (filename_in, "-rewrite.dwg");
+        }
     }
   if (!filename_out || !strcmp (filename_in, filename_out))
     {
-      if (filename_out != argv[i + 1])
+      if (free_fnout)
         free (filename_out);
       return usage ();
     }
@@ -221,7 +234,7 @@ main (int argc, char *argv[])
    * some very simple testing
    */
   printf ("Reading DWG file %s\n", filename_in);
-  error = dwg_read_file (filename_in, &dwg);
+  error = dwg_read_file (filename_in, &dwg); /* 1st read */
   if (error >= DWG_ERR_CRITICAL)
     fprintf (stderr, "READ ERROR 0x%x\n", error);
   num_objects = dwg.num_objects;
@@ -229,15 +242,17 @@ main (int argc, char *argv[])
     {
       printf ("Read 0 objects\n");
       if (error >= DWG_ERR_CRITICAL)
-        return error;
+        {
+          if (free_fnout)
+            free (filename_out);
+          dwg_free (&dwg);
+          return error;
+        }
     }
 
-  if (opts)
-    printf ("\n");
+  //if (opts)
+  //  printf ("\n");
   printf ("Writing DWG file %s", filename_out);
-#ifndef USE_WRITE
-  error = 1;
-#else
   if (version)
     { // forced -as-rXXX
       printf (" as %s\n", version);
@@ -256,28 +271,56 @@ main (int argc, char *argv[])
     {
       printf ("\n");
     }
-  error = dwg_write_file (filename_out, &dwg);
+
+  {
+    struct stat attrib;
+    if (!stat (filename_out, &attrib)) // exists
+      {
+        if (S_ISREG (attrib.st_mode) &&        // refuse to remove a directory
+            (access (filename_out, W_OK) == 0) // is writable
+#ifndef _WIN32
+            // refuse to remove a symlink. even with overwrite. security
+            && !S_ISLNK (attrib.st_mode)
 #endif
+            )
+          unlink (filename_out);
+        else
+          {
+            fprintf (stderr, "ERROR: Not writable file or symlink: %s\n",
+                     filename_out);
+            error |= DWG_ERR_IOERROR;
+          }
+      }
+  }
+
+  error = dwg_write_file (filename_out, &dwg);
   if (error >= DWG_ERR_CRITICAL)
     {
       printf ("WRITE ERROR 0x%x\n", error);
+      if (free_fnout)
+        free (filename_out);
+      dwg_free (&dwg);
       return error;
     }
-  dwg_free (&dwg); // this is slow, but on needed on low memory systems
+  dwg_free (&dwg);
 
-#ifdef USE_WRITE
   // try to read again
-  if (opts)
-    printf ("\n");
+  //if (opts)
+  //  printf ("\n");
   printf ("Re-reading created file %s\n", filename_out);
-  error = dwg_read_file (filename_out, &dwg);
+  error = dwg_read_file (filename_out, &dwg); /* 2nd read */
   if (error >= DWG_ERR_CRITICAL)
     printf ("re-READ ERROR 0x%x\n", error);
   if (num_objects && (num_objects != dwg.num_objects))
     printf ("re-READ num_objects: %lu, should be %lu\n",
             (unsigned long)dwg.num_objects, (unsigned long)num_objects);
+  dwg_free (&dwg);
+
+#ifdef __AFL_HAVE_MANUAL_CONTROL
+  }
 #endif
-  // if (filename_out != argv[2])
-  //  free (filename_out);
+
+  if (free_fnout)
+    free (filename_out);
   return error >= DWG_ERR_CRITICAL ? error : 0;
 }
