@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-# Copyright (C) 2019 Free Software Foundation, Inc., GPL3
+# Copyright (C) 2019-2020 Free Software Foundation, Inc., GPL3
 #
 # Generate src/dynapi.c and test/unit-testing/dynapi_test.c
 # C structs/arrays for all dwg objects and its fields for a dynamic API.
@@ -158,14 +158,14 @@ sub dxf_in {
     $f = '';
     s/DXF \{ //;
     if (!$n) {
-      if (/^DWG_(ENTITY|OBJECT)\((\w+)\)/) {
+      if (/^DWG_(ENTITY|OBJECT)\s?\((\w+)\)/) {
         $n = $2;
         $n =~ s/^_3/3/;
         warn $n;
-      } elsif (/^\s*SECTION\(HEADER\)/) {
+      } elsif (/^\s*SECTION\s?\(HEADER\)/) {
         $n = 'header_variables';
         warn $n;
-      } elsif (/^int DWG_FUNC_N\(ACTION,_HATCH(\w+)\)/) {
+      } elsif (/^int DWG_FUNC_N\s?\(ACTION,_HATCH(\w+)\)/) {
         $n = 'HATCH';
         warn $n;
       } elsif (/^\#define (COMMON_ENTITY_(?:POLYLINE|DIMENSION|))/) {
@@ -179,7 +179,7 @@ sub dxf_in {
       $n = $1;
       warn "define $n";
     # i.e. after #define
-    } elsif (/^DWG_(ENTITY|OBJECT)\((\w+)\)/) {
+    } elsif (/^DWG_(ENTITY|OBJECT)\s?\((\w+)\)/) {
       $n = $2;
       $n =~ s/^_3/3/;
       warn $n;
@@ -276,7 +276,7 @@ sub dxf_in {
       my $type = $1;
       $f = $2;
       $DXF{$n}->{$f} = $3 if $3;
-    } elsif (/^\s+HANDLE_VECTOR\s*\((\w+),.*?,\s*(\d+)\)/) {
+    } elsif (/^\s+HANDLE_VECTOR(?:_N)?\s*\((\w+),.*?,\s*(\d+)\)/) {
       $f = $1;
       $DXF{$n}->{$f} = $2 if $2;
     } elsif (/^\s+HEADER_.+\s*\((\w+),\s*(\d+)\)/) { # HEADER_RC
@@ -798,7 +798,7 @@ print $inc <<"EOF";
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2019 Free Software Foundation, Inc.                        */
+/*  Copyright (C) 2019-2020 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -927,7 +927,7 @@ EOF
     for my $name (@entity_names, @object_names) {
       my $xname = $name =~ /^3/ ? "_$name" : $name; # 3DFACE, 3DSOLID
       #next if $name eq 'DIMENSION_';
-      next if $name eq 'PROXY_LWPOLYLINE';
+      next if $name =~ /^(PROXY_LWPOLYLINE|UNKNOWN_)/;
       print $fh "  else" if $name ne '3DFACE'; # the first
       print $fh <<"EOF";
   if (obj->fixedtype == DWG_TYPE_$xname)
@@ -938,7 +938,8 @@ EOF
   if (m{/\* \@\@for test_OBJECT\@\@ \*/}) {
     for my $name (@entity_names, @object_names) {
       #next if $name eq 'DIMENSION_';
-      next if $name eq 'PROXY_LWPOLYLINE';
+      #TABLE is stored as fixedtype UNKNOWN_ENT, so the dynapi test would fail
+      next if $name =~ /^(PROXY_LWPOLYLINE|UNKNOWN_)/;
       my $is_ent = grep { $name eq $_ } @entity_names;
       my ($Entity, $lentity) = $is_ent ? ('Entity', 'entity') : ('Object', 'object');
       my $xname = $name =~ /^3/ ? "_$name" : $name;
@@ -976,6 +977,7 @@ EOF
     $type =~ s/D_1$/D/;
     $type = 'BITCODE_'.$type unless ($type =~ /^(struct|Dwg_)/ or $type =~ /^[a-z]/);
     if (!$is_ptr) {
+      # TODO DEBUGGING [BR]D can be nan
       print $fh <<"EOF";
   {
     $type $var;
@@ -1013,8 +1015,9 @@ EOF
         #inserts => 'num_inserts',
         #groups => 'num_groups',
         #field_handles => 'num_fields',
-        sort_handles => 'num_ents',
+        sort_ents => 'num_ents',
         attr_def_id => 'num_attr_defs',
+        layer_entries => 'num_entries',
         readdeps  => 'num_deps',
         writedeps => 'num_deps',
         dashes_r11 => 'num_dashes',
@@ -1069,7 +1072,10 @@ EOF
     if (dwg_dynapi_entity_value ($lname, "$name", "$countfield", &count, NULL)
         && dwg_dynapi_entity_value ($lname, "$name", "$var", &$svar, NULL)
 EOF
-        if ($type !~ /\*\*/) {
+        if ($type eq 'BITCODE_BD') {
+          print $fh "        && (isnan ($svar) || $svar == $lname->$svar)";
+        }
+        elsif ($type !~ /\*\*/) {
           print $fh "        && $svar == $lname->$svar";
         }
         print $fh ")\n";
@@ -1124,7 +1130,7 @@ EOF
     for my $name (@entity_names, @object_names) {
       my $xname = $name =~ /^3/ ? "_$name" : $name; # 3DFACE, 3DSOLID
       next if $name eq 'DIMENSION_';
-      next if $name eq 'PROXY_LWPOLYLINE';
+      next if $name =~ /^(PROXY_LWPOLYLINE|UNKNOWN_)/;
       print $fh "  else" if $name ne '3DFACE'; # the first
       print $fh <<"EOF";
   if (obj->fixedtype == DWG_TYPE_$xname)
@@ -1137,13 +1143,14 @@ close $in;
 chmod 0444, $fh;
 close $fh;
 
+# NOTE: in the 2 #line's below use __LINE__
 __DATA__
 /* ex: set ro ft=c: -*- mode: c; buffer-read-only: t -*- */
-#line 1141 "gen-dynapi.pl"
+#line 1149 "gen-dynapi.pl"
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2018-2019 Free Software Foundation, Inc.                   */
+/*  Copyright (C) 2018-2020 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -1220,7 +1227,7 @@ static const struct _name_subclass_fields dwg_list_subclasses[] = {
 @@list subclasses@@
 };
 
-#line 1222 "gen-dynapi.pl"
+#line 1230 "gen-dynapi.pl"
 static int
 _name_inl_cmp (const void *restrict key, const void *restrict elem)
 {
@@ -1367,7 +1374,7 @@ dwg_dynapi_entity_value (void *restrict _obj, const char *restrict name,
     const Dwg_Object* obj = dwg_obj_generic_to_object (_obj, &error);
     if (obj && strNE (obj->name, name)) // objid may be 0
       {
-        const int loglevel = obj->parent->opts & 0xf;
+        const int loglevel = obj->parent->opts & DWG_OPTS_LOGLEVEL;
         LOG_ERROR ("%s: Invalid entity type %s, wanted %s", __FUNCTION__,
                    obj->name, name);
         return false;
@@ -1378,7 +1385,7 @@ dwg_dynapi_entity_value (void *restrict _obj, const char *restrict name,
         {
           int loglevel;
           if (obj)
-            loglevel = obj->parent->opts & 0xf;
+            loglevel = obj->parent->opts & DWG_OPTS_LOGLEVEL;
           else
             loglevel = DWG_LOGLEVEL_ERROR;
           LOG_ERROR ("%s: Invalid %s field %s", __FUNCTION__, name, fieldname);
@@ -1395,8 +1402,11 @@ dwg_dynapi_entity_value (void *restrict _obj, const char *restrict name,
 EXPORT bool
 dwg_dynapi_entity_utf8text (void *restrict _obj, const char *restrict name,
                             const char *restrict fieldname,
-                            char **restrict out, Dwg_DYNAPI_field *restrict fp)
+                            char **restrict out, int *isnew,
+                            Dwg_DYNAPI_field *restrict fp)
 {
+  if (isnew)
+    *isnew = 0;
 #ifndef HAVE_NONNULL
   if (!_obj || !name || !fieldname || !out)
     return false;
@@ -1406,7 +1416,7 @@ dwg_dynapi_entity_utf8text (void *restrict _obj, const char *restrict name,
     const Dwg_Object* obj = dwg_obj_generic_to_object (_obj, &error);
     if (obj && strNE (obj->name, name)) // objid may be 0
       {
-        const int loglevel = obj->parent->opts & 0xf;
+        const int loglevel = obj->parent->opts & DWG_OPTS_LOGLEVEL;
         LOG_ERROR ("%s: Invalid entity type %s, wanted %s", __FUNCTION__,
                    obj->name, name);
         return false;
@@ -1419,7 +1429,7 @@ dwg_dynapi_entity_utf8text (void *restrict _obj, const char *restrict name,
         {
           int loglevel;
           if (obj)
-            loglevel = obj->parent->opts & 0xf;
+            loglevel = obj->parent->opts & DWG_OPTS_LOGLEVEL;
           else
             loglevel = DWG_LOGLEVEL_ERROR;
           LOG_ERROR ("%s: Invalid %s text field %s", __FUNCTION__, name, fieldname);
@@ -1435,6 +1445,8 @@ dwg_dynapi_entity_utf8text (void *restrict _obj, const char *restrict name,
           if (!utf8) // some conversion error, invalid wchar (nyi)
             return false;
           *out = utf8;
+          if (isnew)
+            *isnew = 1;
         }
       else
         {
@@ -1468,7 +1480,7 @@ dwg_dynapi_header_value (const Dwg_Data *restrict dwg,
       }
     else
       {
-        const int loglevel = dwg->opts & 0xf;
+        const int loglevel = dwg->opts & DWG_OPTS_LOGLEVEL;
         LOG_ERROR ("%s: Invalid header field %s", __FUNCTION__, fieldname);
         return false;
       }
@@ -1478,9 +1490,11 @@ dwg_dynapi_header_value (const Dwg_Data *restrict dwg,
 EXPORT bool
 dwg_dynapi_header_utf8text (const Dwg_Data *restrict dwg,
                             const char *restrict fieldname,
-                            char **restrict out,
+                            char **restrict out, int *isnew,
                             Dwg_DYNAPI_field *restrict fp)
 {
+  if (isnew)
+    *isnew = 0;
 #ifndef HAVE_NONNULL
   if (!dwg || !fieldname || !out)
     return false;
@@ -1502,6 +1516,8 @@ dwg_dynapi_header_utf8text (const Dwg_Data *restrict dwg,
             if (!utf8) // some conversion error, invalid wchar (nyi)
               return false;
             *out = utf8;
+            if (isnew)
+              *isnew = 1;
           }
         else
           {
@@ -1513,7 +1529,7 @@ dwg_dynapi_header_utf8text (const Dwg_Data *restrict dwg,
       }
     else
       {
-        const int loglevel = dwg->opts & 0xf;
+        const int loglevel = dwg->opts & DWG_OPTS_LOGLEVEL;
         LOG_ERROR ("%s: Invalid header text field %s", __FUNCTION__, fieldname);
         return false;
       }
@@ -1551,7 +1567,7 @@ dwg_dynapi_common_value(void *restrict _obj, const char *restrict fieldname,
       }
     else
       {
-        const int loglevel = obj->parent->opts & 0xf; // DWG_LOGLEVEL_ERROR;
+        const int loglevel = obj->parent->opts & DWG_OPTS_LOGLEVEL; // DWG_LOGLEVEL_ERROR;
         LOG_ERROR ("%s: Unhandled %s.supertype ", __FUNCTION__, obj->name);
         return false;
       }
@@ -1565,7 +1581,7 @@ dwg_dynapi_common_value(void *restrict _obj, const char *restrict fieldname,
       }
     else
       {
-        const int loglevel = obj->parent->opts & 0xf;
+        const int loglevel = obj->parent->opts & DWG_OPTS_LOGLEVEL;
         LOG_ERROR ("%s: Invalid common field %s", __FUNCTION__, fieldname);
         return false;
       }
@@ -1574,8 +1590,10 @@ dwg_dynapi_common_value(void *restrict _obj, const char *restrict fieldname,
 
 EXPORT bool
 dwg_dynapi_common_utf8text(void *restrict _obj, const char *restrict fieldname,
-                           char **restrict out, Dwg_DYNAPI_field *restrict fp)
+                           char **restrict out, int *isnew, Dwg_DYNAPI_field *restrict fp)
 {
+  if (isnew)
+    *isnew = 0;
 #ifndef HAVE_NONNULL
   if (!_obj || !fieldname || !out)
     return false;
@@ -1631,6 +1649,8 @@ dwg_dynapi_common_utf8text(void *restrict _obj, const char *restrict fieldname,
             if (!utf8) // some conversion error, invalid wchar (nyi)
               return false;
             *out = utf8;
+            if (isnew)
+              *isnew = 1;
           }
         else
           {
@@ -1642,7 +1662,7 @@ dwg_dynapi_common_utf8text(void *restrict _obj, const char *restrict fieldname,
       }
     else
       {
-        const int loglevel = dwg->opts & 0xf;
+        const int loglevel = dwg->opts & DWG_OPTS_LOGLEVEL;
         LOG_ERROR ("%s: Invalid common text field %s", __FUNCTION__, fieldname);
         return false;
       }
@@ -1709,7 +1729,7 @@ dwg_dynapi_entity_set_value (void *restrict _obj, const char *restrict name,
     const Dwg_Object *obj = dwg_obj_generic_to_object (_obj, &error);
     if (obj && strNE (obj->name, name))
       {
-        const int loglevel = obj->parent->opts & 0xf;
+        const int loglevel = obj->parent->opts & DWG_OPTS_LOGLEVEL;
         LOG_ERROR ("%s: Invalid entity type %s, wanted %s", __FUNCTION__,
                    obj->name, name);
         return false;
@@ -1724,7 +1744,7 @@ dwg_dynapi_entity_set_value (void *restrict _obj, const char *restrict name,
 
       if (!f)
         {
-          const int loglevel = dwg->opts & 0xf;
+          const int loglevel = dwg->opts & DWG_OPTS_LOGLEVEL;
           LOG_ERROR ("%s: Invalid %s field %s", __FUNCTION__, name, fieldname);
           return false;
         }
@@ -1762,7 +1782,7 @@ dwg_dynapi_header_set_value (const Dwg_Data *restrict dwg,
       }
     else
       {
-        const int loglevel = dwg->opts & 0xf;
+        const int loglevel = dwg->opts & DWG_OPTS_LOGLEVEL;
         LOG_ERROR ("%s: Invalid header field %s", __FUNCTION__, fieldname);
         return false;
       }
@@ -1814,7 +1834,7 @@ dwg_dynapi_common_set_value (void *restrict _obj,
 
     if (!f)
       {
-        const int loglevel = obj->parent->opts & 0xf;
+        const int loglevel = obj->parent->opts & DWG_OPTS_LOGLEVEL;
         LOG_ERROR ("%s: Invalid %s common field %s", __FUNCTION__, obj->name, fieldname);
         return false;
       }
