@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2018-2019 Free Software Foundation, Inc.                   */
+/*  Copyright (C) 2018-2020 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -132,16 +132,18 @@ static unsigned int cur_ver = 0;
     const int len = strlen (str);                                             \
     if (len < 4096 / 6)                                                       \
       {                                                                       \
-        char *_buf = alloca (6 * len + 1);                                    \
+        const int _len = 6 * len + 1;                                         \
+        char *_buf = alloca (_len);                                           \
         PREFIX fprintf (dat->fh, "\"" #name "\": \"%s\",\n",                  \
-                        json_cquote (_buf, str));                             \
+                        json_cquote (_buf, str, _len));                       \
         freea (_buf);                                                         \
       }                                                                       \
     else                                                                      \
       {                                                                       \
-        char *_buf = malloc (6 * len + 1);                                    \
+        const int _len = 6 * len + 1;                                         \
+        char *_buf = malloc (_len);                                           \
         PREFIX fprintf (dat->fh, "\"" #name "\": \"%s\",\n",                  \
-                        json_cquote (_buf, str));                             \
+                        json_cquote (_buf, str, _len));                       \
         free (_buf);                                                          \
       }                                                                       \
   }
@@ -321,7 +323,7 @@ static unsigned int cur_ver = 0;
 #define WARN_UNSTABLE_CLASS                                                   \
   LOG_WARN ("Unstable Class %s %d %s (0x%x%s) -@%ld",                         \
             is_entity ? "entity" : "object", klass->number, dxfname,          \
-            klass->proxyflag, klass->wasazombie ? " was proxy" : "",          \
+            klass->proxyflag, klass->is_zombie ? "is_zombie" : "",          \
             obj->address + obj->size)
 
 // common properties
@@ -754,7 +756,7 @@ geojson_entities_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 EXPORT int
 dwg_write_geojson (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 {
-  // const int minimal = dwg->opts & 0x10;
+  // const int minimal = dwg->opts & DWG_OPTS_MINIMAL;
   char date[12] = "YYYY-MM-DD";
   time_t rawtime;
 
